from .parameter_dict import ParameterDict
import requests
import json
import datetime
import pandas as pd


class LiquidPublic:
    def __init__(self):
        self.end_point = "https://api.liquid.com/products/"
        self.parameter_dict = ParameterDict()

    def get_candlestick(self, currency_name, date, candle_type):
        """
        ローソク足を取得
        :param currency_name: 通貨名
        :param date: 取得対象日付
        :param candle_type: ローソク足範囲
        - 1min
        - 5min
        - 15min
        - 30min
        - 1hour
        :return: ローソク足DataFrame
        - datetime: yyyy-mm-dd hh:mm:ss
        - open
        - high
        - low
        - close
        - volume
        """
        url = self.end_point

        # 通貨名の不正チェック
        if currency_name in self.parameter_dict.name2id:
            url += self.parameter_dict.name2id[currency_name]
        else:
            raise ValueError("通貨名が不正です。")
        url += "/ohlc"

        # ローソクタイプの不正チェック
        if candle_type in self.parameter_dict.resolution2id:
            # URLにresolutionを付与
            second = self.parameter_dict.resolution2id[candle_type]
            query = "?resolution=" + second + "&limit=800"
        else:
            raise ValueError("ローソク足のタイプが不正です。")

        url += query
        req_result = requests.get(url).text
        parsed_data = json.loads(req_result)["data"]

        output_df = pd.DataFrame(data=parsed_data, columns=["datetime", "open", "high", "low", "close", "volume"])
        output_df["datetime"] = output_df["datetime"].apply(datetime.datetime.fromtimestamp)

        # 引数のdateをdatetime型に変換
        target_date = datetime.datetime.strptime(date, "%Y%m%d")
        target_date_next = target_date + datetime.timedelta(days=1)

        # 引数で指定したデータを取得
        output_df = output_df.loc[(output_df["datetime"] >= target_date) & (output_df["datetime"] < target_date_next)]
        output_df = output_df.reset_index(drop=True)

        if len(output_df) == 0:
            raise ValueError("指定日付が取得範囲外です。")

        return output_df

    def get_order_book(self, currency_name):
        """
        板情報を取得
        :param currency_name: 通貨名
        :return: 売値DataFrame, 買値DataFrame, datetime
        """
        url = self.end_point
        # 通貨名の不正チェック
        if currency_name in self.parameter_dict.name2id:
            url += self.parameter_dict.name2id[currency_name]
        else:
            raise ValueError("通貨名が不正です。")
        url += "/price_levels"
        query = "?full=0"
        url += query
        req_result = requests.get(url).text
        parsed_data = json.loads(req_result)

        # 売値
        bid_df = pd.DataFrame(data=parsed_data["sell_price_levels"], columns=["bid_price", "bid_volume"])

        # 買値
        ask_df = pd.DataFrame(data=parsed_data["buy_price_levels"], columns=["ask_price", "ask_volume"])

        # 板情報取得日時
        datetime_data = datetime.datetime.fromtimestamp(float(parsed_data["timestamp"]))
        return bid_df, ask_df, datetime_data



import jwt
import json
import requests
from datetime import datetime
import warnings

from .parameter_dict import ParameterDict


class LiquidPrivate:
    def __init__(self, token_id, secret_key):
        """
        :param token_id:
        :param secret_key:
        """
        self.token_id = token_id
        self.secret_key = secret_key

        self.endpoint = "https://api.liquid.com/"
        self.parameter_dict = ParameterDict()

    def make_header(self, path, query=""):
        timestamp = datetime.now().timestamp()
        path += query

        payload_data = {
            "path": path,
            "nonce": timestamp,
            "token_id": self.token_id
        }
        signature = jwt.encode(payload_data, self.secret_key, algorithm="HS256")
        header = {
            "X-Quoine-API-Version": "2",
            "X-Quoine-Auth": signature,
            "Content-Type": "application/json"
        }

        return path, header

    def check_and_trans_params(self, currency_name, side, amount, price):
        currency_id = None

        if currency_name is not None:
            try:
                currency_id = self.parameter_dict.name2id[currency_name]
            except KeyError:
                raise ValueError("通貨名が不正です。")

        if side is not None:
            if side not in self.parameter_dict.side_list:
                raise ValueError("注文にはbuyもしくはsellを指定してください。")

        amount = float(amount)
        price = float(price)

        return currency_id, amount, price

    def create_order(self, currency_name, side, amount, price=0.0, order_type="limit"):
        """
        注文
        :param currency_name: 通貨名
        :param side: 売/買
        - sell: 売り注文
        - buy: 買い注文
        :param amount: 注文量
        :param price: 注文単価
        :param order_type: 注文タイプ
        - limit: 指値注文
        - market: 成行注文
        :return:
        """
        url = self.endpoint + "orders/"
        currency_id, amount, price = self.check_and_trans_params(
            currency_name=currency_name,
            side=side,
            amount=amount,
            price=price
        )

        # ヘッダ情報作成
        url, header = self.make_header(path=url)

        # 注文データ
        send_data = {
            "order": {
                "order_type": order_type,
                "product_id": currency_id,
                "side": side,
                "quantity": amount,
                "price": price,
            }
        }

        # 送信データ作成
        json_data = json.dumps(send_data)
        res = requests.post(url=url, headers=header, data=json_data)

        parsed_data = json.loads(res.text)
        create_datetime = datetime.fromtimestamp(parsed_data["created_at"])
        create_datetime = datetime.strftime(create_datetime, "%Y/%m/%d %H:%M:%S")

        update_datetime = datetime.fromtimestamp(parsed_data["updated_at"])
        update_datetime = datetime.strftime(update_datetime, "%Y/%m/%d %H:%M:%S")

        output = {
            "transaction_id": str(parsed_data["id"]),
            "order_type": parsed_data["order_type"],
            "quantity": float(parsed_data["quantity"]),
            "price": float(parsed_data["price"]),
            "side": parsed_data["side"],
            "created_at": create_datetime,
            "updated_at": update_datetime,
            "currency": parsed_data["currency_pair_code"][:-3]  # 取引通貨のみ取得（文字列からJPYを除く）
        }

        return output

    def get_order_info(self, limit_num=None):
        url = self.endpoint + "orders"
        query = None

        # 取得件数を指定した場合はクエリを設定
        if limit_num is not None:
            query = "?limit=" + str(limit_num)

        # ヘッダ情報作成
        url, header = self.make_header(path=url, query=query)
        # データ送信
        res = requests.get(url=url, headers=header)

        if res.status_code != 200:
            print("注文情報の取得に失敗しました。")
            return None

        parsed_data = json.loads(res.text)["models"]
        output_list = []
        for data in parsed_data:
            create_datetime = datetime.fromtimestamp(data["created_at"])
            create_datetime = datetime.strftime(create_datetime, "%Y/%m/%d %H:%M:%S")

            update_datetime = datetime.fromtimestamp(data["updated_at"])
            update_datetime = datetime.strftime(update_datetime, "%Y/%m/%d %H:%M:%S")

            # 出力データ作成
            output_dict = {
                "transaction_id": str(data["id"]),
                "order_type": data["order_type"],
                "price": float(data["price"]),
                "quantity": float(data["quantity"]),
                "filled_quantity": float(data["filled_quantity"]),  # 約定済の量
                "side": data["side"],
                "created_at": create_datetime,
                "updated_at": update_datetime,
                "currency": data["currency_pair_code"][:-3]  # 取引通貨のみ取得（文字列からJPYを除く）
            }

            output_list.append(output_dict)

        return output_list

    def cancel_order(self, order_id):
        url = self.endpoint + "orders/" + order_id + "/cancel"
        # ヘッダ情報作成
        header = self.make_header(path=url)
        # データ送信
        res = requests.put(url=url, headers=header)

        if res.status_code == 404:
            print("対象の取引IDが存在しません。取引ID:", order_id)
        elif res.status_code == 200:
            print("注文がキャンセルされました。取引ID:", order_id)

    def get_fiat_info(self):
        """
        日本円残高取得
        :return:
        - balanced: 利用可能残高
        - reserved: ロック中残高
        """
        warnings.warn("廃止予定のメソッドです。get_asset_info(asset='jpy')を使用してください。", category=FutureWarning)
        url = self.endpoint + "fiat_accounts"
        # ヘッダ情報作成
        url, header = self.make_header(path=url)
        # データ送信
        res = requests.get(url=url, headers=header)
        parsed = json.loads(res.text)[0]

        balance = parsed["balance"]  # 日本円残高
        reserved = parsed["reserved_balance"]  # ロック中
        return balance, reserved

    def get_crypto_info(self, currency="btc"):
        warnings.warn("廃止予定のメソッドです。get_asset_info(asset)を使用してください。", category=FutureWarning)
        url = self.endpoint + "crypto_accounts"
        # ヘッダ情報作成
        url, header = self.make_header(path=url)
        # データ送信
        res = requests.get(url=url, headers=header)
        parsed = json.loads(res.text)

        currency = currency.upper()
        data = None
        # 引数で与えた通貨の情報を取得
        for p in parsed:
            if p["currency"] == currency:
                data = p
                break

        balance = data["balance"]  # 利用可能残高
        reserved = data["reserved_balance"]  # ロック中残高

        return balance, reserved

    def get_asset_info(self, asset):
        url = self.endpoint
        # 日本円を指定した場合
        if asset in self.parameter_dict.fiat_list:
            url += "fiat_accounts"
        # 暗号資産を指定した場合
        elif asset in self.parameter_dict.name2id:
            url += "crypto_accounts"
        else:
            raise Exception("通貨名が不正です。")

        # ヘッダ情報作成
        url, header = self.make_header(path=url)
        # データ送信
        res = requests.get(url=url, headers=header)
        parsed = json.loads(res.text)
        asset = asset.upper()

        data = None
        # 引数で与えた通貨の情報を取得
        for p in parsed:
            if p["currency"] == asset:
                data = p
                break

        balance = data["balance"]  # 利用可能残高
        reserved = data["reserved_balance"]  # ロック中残高

        return balance, reserved



from setuptools import setup
import python_liquid_api


NAME = "python_liquid_api_tths"
AUTHOR = "kitataku"
EMAIL = ""
DESCRIPTION = "Use Liquid API with Python"
CONTENT_TYPE = "text/markdown"
LICENSE = "MIT License"
URL = "https://github.com/kitataku/python_liquid_api_tths"
VERSION = python_liquid_api.__version__
PYTHON_REQUIRES = ">=3.6"

INSTALL_REQUIRES = [
    "PyJWT>=2.1.0",
    "pandas>=1.3.4",
    "numpy>=1.20.3",
]

PACKAGES = [
    "python_liquid_api"
]

with open("README.md", "r") as f:
    readme = f.read()

long_description = readme

setup(
    name=NAME,
    author=AUTHOR,
    author_email=EMAIL,
    maintainer=AUTHOR,
    maintainer_email=EMAIL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type=CONTENT_TYPE,
    licence=LICENSE,
    url=URL,
    version=VERSION,
    python_requires=PYTHON_REQUIRES,
    install_requires=INSTALL_REQUIRES,
    packages=PACKAGES
)
